﻿//////////////////////////////////////////////
// MeshInputLayout.h
//
//////////////////////////////////////////////

/// Defines / Macros -------------------------

#pragma once

/// Includes ---------------------------------

// nkGraphics
#include "../../Dll/DllDefines.h"

#include "MeshInputLayoutAttribute.h"

// nkMemory
#include <NilkinsMemory/Containers/BufferCast.h>
#include <NilkinsMemory/Containers/String.h>
#include <NilkinsMemory/Containers/StringView.h>

/// Class ------------------------------------

namespace nkGraphics
{
	class DLL_GRAPHICS_EXPORT MeshInputLayout final
	{
		private :

			// Structures
			struct AttributeEntry final
			{
				unsigned long long _hash = 0ull ;
				MeshInputLayoutAttribute _attribute ;
			} ;

		private :

			// Attributes
			// List of attributes inside
			nkMemory::BufferCast<AttributeEntry> _attributes ;

			// Annotations
			nkMemory::String _positionAttributeName ;
			nkMemory::String _normalAttributeName ;
			nkMemory::String _texCoordAttributeName ;

			// Full hash for quicker sorting and comparison
			unsigned long long _hash ;

		private :

			// Functions
			// Hash
			void _recomputeHash () ;

		public :

			// Constructor, destructor
			MeshInputLayout () ;
			~MeshInputLayout () ;

			// Getters
			unsigned long long getHash () const ;
			nkMemory::StringView getPositionAttributeName () const ;
			nkMemory::StringView getNormalAttributeName () const ;
			nkMemory::StringView getTexCoordAttributeName () const ;

			// Setters
			void setAttribute (const MeshInputLayoutAttribute& attribute, unsigned int index) ;
			void setPositionAttributeName (const nkMemory::StringView& value) ;
			void setNormalAttributeName (const nkMemory::StringView& value) ;
			void setTexCoordAttributeName (const nkMemory::StringView& value) ;

			// Attribute management
			void addAttribute (const MeshInputLayoutAttribute& attribute) ;
			const MeshInputLayoutAttribute* getAttribute (unsigned int index) const ;
			const MeshInputLayoutAttribute* getAttribute (const nkMemory::StringView& name) const ;
			unsigned int getAttributeCount () const ;
			unsigned int getTotalDeclarationSlots () const ;
			void removeAttribute (unsigned int index) ;
			void removeAttribute (const nkMemory::StringView& name) ;
			bool hasAttribute (const nkMemory::StringView& name) const ;

			// Operators
			bool operator< (const MeshInputLayout& other) const ;
			bool operator== (const MeshInputLayout& other) const ;
	} ;
}