﻿//////////////////////////////////////////////
// DepthStencilStateManager.h
//
//////////////////////////////////////////////

/// Defines / Macros -------------------------

#pragma once

/// Forward decl -----------------------------

namespace nkGraphics
{
	class DepthStencilState ;
}

/// Includes ---------------------------------

// nkGraphics
#include "../Dll/DllDefines.h"

// nkCommon
#include <nilkinsCommon/Patterns/SingletonClass.h>

// nkMemory
#include <NilkinsMemory/Containers/StringView.h>

// Standards
#include <mutex>
#include <string>
#include <unordered_map>

/// Class ------------------------------------

namespace nkGraphics
{
	class DLL_GRAPHICS_EXPORT DepthStencilStateManager : public nkCommon::SingletonClass<DepthStencilStateManager>
	{
		friend class nkCommon::SingletonClass<DepthStencilStateManager> ;

		private :
		
			// Attributs
			// Liste des states
			std::unordered_map<std::string, DepthStencilState*> _memory ;

			// State par défaut
			DepthStencilState* _defaultState ;

			// Threading
			std::mutex _mutex ;

		private :

			// Functions
			// Constructeur
			DepthStencilStateManager () ;
			
		public :
		
			// Destructeur
			~DepthStencilStateManager () ;

			// Manipulation mémoire
			DepthStencilState* createOrRetrieve (const nkMemory::StringView& name) ;
			DepthStencilState* get (const nkMemory::StringView& name) ;
			DepthStencilState* getByIndex (unsigned int index) ;
			void rename (const nkMemory::StringView& oldName, const nkMemory::StringView& newName) ;
			void erase (const nkMemory::StringView& name) ;

			// Getters
			DepthStencilState* getDefaultState () const ;
	} ;
}