﻿//////////////////////////////////////////////
// Resource.h
//
// Base pour toute ressource :
// Shader, Mesh, Texture...
//////////////////////////////////////////////

/// Defines / Macros -------------------------

#pragma once

/// Includes ---------------------------------

// nkGraphics
#include "ResourceLoadState.h"
#include "ResourceType.h"

#include "../Dll/DllDefines.h"

// nkMemory
#include <NilkinsMemory/Containers/String.h>
#include <NilkinsMemory/Containers/StringView.h>
	
/// Class ------------------------------------

namespace nkGraphics
{
	class DLL_GRAPHICS_EXPORT Resource
	{
		protected :
		
			// Attributs
			// Loading state
			RESOURCE_LOAD_STATE _loadState ;
			// An identifier to know what kind of resource it is
			RESOURCE_TYPE _resourceTypeName ;

			// Its path and name to identify it
			nkMemory::String _resourcePath ;
			nkMemory::String _resourceName ;

			// Whether it will be hidden during a global export
			bool _hidden ;
			// Whether the resource's loading can be deferred or not before rendering
			bool _gpuUploadCanBeDeferred ;
		
		public :
		
			// Constructor, destructor
			Resource () ;
			Resource (const nkMemory::StringView& path) ;
			virtual ~Resource () ;

			// Getters
			bool isReadyForRendering () const ;
			bool isUnloaded () const ;
			RESOURCE_LOAD_STATE getLoadState () const ;
			nkMemory::StringView getResourcePath () const ;
			RESOURCE_TYPE getResourceTypeName () const ;
			nkMemory::StringView getResourceName () const ;
			bool getHidden () const ;
			bool getGpuUploadCanBeDeferred () const ;

			// Setters
			virtual void setResourcePath (const nkMemory::StringView& path) ;
			void setResourceName (const nkMemory::StringView& name) ;
			void setHidden (bool value) ;
			void setGpuUploadCanBeDeferred (bool value) ;

			// Loading
			virtual bool load () = 0 ;
			virtual void unload () = 0 ;
	} ;
}